/*
 * Decompiled with CFR 0_101.
 */
package org.apache.http.impl.auth;

import org.apache.commons.codec.binary.Base64;
import org.apache.http.impl.auth.NTLMEngineException;
import org.apache.http.impl.auth.NTLMEngineImpl;
import org.apache.http.util.EncodingUtils;

static class NTLMEngineImpl.NTLMMessage {
    private byte[] messageContents = null;
    private int currentOutputPosition = 0;

    NTLMEngineImpl.NTLMMessage() {
    }

    NTLMEngineImpl.NTLMMessage(String messageBody, int expectedType) throws NTLMEngineException {
        this.messageContents = Base64.decodeBase64(EncodingUtils.getBytes(messageBody, "ASCII"));
        if (this.messageContents.length < SIGNATURE.length) {
            throw new NTLMEngineException("NTLM message decoding error - packet too short");
        }
        for (int i = 0; i < SIGNATURE.length; ++i) {
            if (this.messageContents[i] == SIGNATURE[i]) continue;
            throw new NTLMEngineException("NTLM message expected - instead got unrecognized bytes");
        }
        int type = this.readULong(SIGNATURE.length);
        if (type != expectedType) {
            throw new NTLMEngineException("NTLM type " + Integer.toString(expectedType) + " message expected - instead got type " + Integer.toString(type));
        }
        this.currentOutputPosition = this.messageContents.length;
    }

    protected int getPreambleLength() {
        return SIGNATURE.length + 4;
    }

    protected int getMessageLength() {
        return this.currentOutputPosition;
    }

    protected byte readByte(int position) throws NTLMEngineException {
        if (this.messageContents.length < position + 1) {
            throw new NTLMEngineException("NTLM: Message too short");
        }
        return this.messageContents[position];
    }

    protected void readBytes(byte[] buffer, int position) throws NTLMEngineException {
        if (this.messageContents.length < position + buffer.length) {
            throw new NTLMEngineException("NTLM: Message too short");
        }
        System.arraycopy(this.messageContents, position, buffer, 0, buffer.length);
    }

    protected int readUShort(int position) throws NTLMEngineException {
        return NTLMEngineImpl.readUShort(this.messageContents, position);
    }

    protected int readULong(int position) throws NTLMEngineException {
        return NTLMEngineImpl.readULong(this.messageContents, position);
    }

    protected byte[] readSecurityBuffer(int position) throws NTLMEngineException {
        return NTLMEngineImpl.readSecurityBuffer(this.messageContents, position);
    }

    protected void prepareResponse(int maxlength, int messageType) {
        this.messageContents = new byte[maxlength];
        this.currentOutputPosition = 0;
        this.addBytes(SIGNATURE);
        this.addULong(messageType);
    }

    protected void addByte(byte b) {
        this.messageContents[this.currentOutputPosition] = b;
        ++this.currentOutputPosition;
    }

    protected void addBytes(byte[] bytes) {
        for (int i = 0; i < bytes.length; ++i) {
            this.messageContents[this.currentOutputPosition] = bytes[i];
            ++this.currentOutputPosition;
        }
    }

    protected void addUShort(int value) {
        this.addByte((byte)(value & 255));
        this.addByte((byte)(value >> 8 & 255));
    }

    protected void addULong(int value) {
        this.addByte((byte)(value & 255));
        this.addByte((byte)(value >> 8 & 255));
        this.addByte((byte)(value >> 16 & 255));
        this.addByte((byte)(value >> 24 & 255));
    }

    String getResponse() {
        byte[] resp;
        if (this.messageContents.length > this.currentOutputPosition) {
            byte[] tmp = new byte[this.currentOutputPosition];
            for (int i = 0; i < this.currentOutputPosition; ++i) {
                tmp[i] = this.messageContents[i];
            }
            resp = tmp;
        } else {
            resp = this.messageContents;
        }
        return EncodingUtils.getAsciiString(Base64.encodeBase64(resp));
    }
}
